/* Adapted from XviD */
#include <sighandler.h>

#ifdef _MSC_VER

#include <windows.h>

int sigill_check(void (*func)()) {
	_try {
		func();
	} _except(EXCEPTION_EXECUTE_HANDLER) {
		if (_exception_code() == STATUS_ILLEGAL_INSTRUCTION)
			return EXIT_ERR;
	}
	return EXIT_OK;
}

#else

#include <signal.h>
#include <setjmp.h>

static jmp_buf mark;

static void sigill_handler(int signal) {
	longjmp(mark,1);
}

int sigill_check(void (*func)()) {
    volatile void *old_handler;
    volatile int jmpret;

    old_handler = signal(SIGILL, sigill_handler);

    if (old_handler == SIG_ERR) {
        return(-1);
    }

    jmpret=setjmp(mark);

    if(!jmpret) func();

    signal(SIGILL, old_handler);

    if(jmpret) return EXIT_ERR;

    return EXIT_OK;
}

#endif 
